<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Contracts\Validation\Validator;

use App\Models\Cashtransaction;
use App\Models\Subcashtransaction;
use App\Models\Akun;
use App\Models\Credit;
use App\Models\CreditCashTransaction;
use App\Models\Stocktransaction;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Symfony\Contracts\Service\Attribute\Required;

class CashController extends Controller
{
    public function getCash(Request $request){

        $data = Cashtransaction::with('from','to');

        if (!empty($request->start_date) && !empty($request->end_date)) {
            $request->start_date = date('Y-m-d',strtotime($request->start_date));
            $request->end_date = date('Y-m-d',strtotime($request->end_date));
            $data = $data->whereBetween('date',[$request->start_date,$request->end_date]);
        }
        // else{
        //     $data = $data->whereBetween('date',[date('Y-m-01',time()),date('Y-m-d',time())]);
        // }

        $data =$data->get();
        
        $response = [
            'success'=>true,
            'cashtransaction'=>$data,

        ];
        
        return response($response,200);
    }

    public function getCashIn(Request $request){
        $data = Cashtransaction::whereNotNull('cashin')->whereNull('from');
    
        if (!empty($request->start_date) && !empty($request->end_date)) {
            $request->start_date = date('Y-m-d',strtotime($request->start_date));
            $request->end_date = date('Y-m-d',strtotime($request->end_date));
            $data = $data->whereBetween('date',[$request->start_date,$request->end_date]);
        }
        // else{
        //     $data = $data->whereBetween('date',[date('Y-m-01',time()),date('Y-m-d',time())]);
        // }
    
        if($request->type){
            // mengambil data berdasarkan filter
            $data = $data->where('transaction_category', $request->type)->with('to','subcashtransaction.akun')->orderBy('id', 'desc')->get();
        }else{
            // mengambil semua data
            $data = $data->with('to','subcashtransaction.akun')->orderBy('id', 'desc')->get();
        }
    
        // map data and return transaction_type
        $data = $data->map(function ($item) {
            // Map untuk transaction_name
            $transactionName = $item->subcashtransaction->isNotEmpty()
                ? $item->subcashtransaction->filter(function ($subItem) use ($item) {
                    return $item->id == $subItem->cashtransaction_id;
                })->map(function ($subItem) {
                    return $subItem->akun->name;
                })->first()
                : 'No Name';
    
            $childrenTotal = Cashtransaction::where('pay_debt_to', $item->id)->sum('cashout');
    
            $item->transaction_name = $transactionName;
            $item->paid = $childrenTotal;
    
            return $item;
        });
    
        $response = [
            'success'=>true,
            'cashtransaction'  => $data,
        ];
    
        return response($response,200);
    }    

    public function getCashOut(Request $request){

        $data = Cashtransaction::whereNotNull('cashout')->whereNull('to');
        if (!empty($request->start_date) && !empty($request->end_date)) {
            $request->start_date = date('Y-m-d',strtotime($request->start_date));
            $request->end_date = date('Y-m-d',strtotime($request->end_date));
            $data = $data->whereBetween('date',[$request->start_date,$request->end_date]);
        }
        // else{
        //     $data = $data->whereBetween('date',[date('Y-m-01',time()),date('Y-m-d',time())]);
        // }
        
        if($request->type){
            // mengambil data berdasarkan filter
            $data = $data->where('transaction_category', $request->type)->with('from','contact','subcashtransaction.akun')->orderBy('id', 'desc')->get();
        }else{
            // mengambil semua data
            $data =$data->with('from','contact','subcashtransaction.akun')->orderBy('id', 'desc')->get();
        }
        // map data and return transaction_type
        $data = $data->map(function ($item) {
            // Map untuk transaction_name
            $transactionName = $item->subcashtransaction->isNotEmpty()
                ? $item->subcashtransaction->filter(function ($subItem) use ($item) {
                    return $item->id == $subItem->cashtransaction_id;
                })->map(function ($subItem) {
                    return $subItem->akun->name;
                })->first()
                : 'No Name';
    
            $childrenTotal = Cashtransaction::where('receivable_id', $item->id)->sum('cashin');
    
            $item->transaction_name = $transactionName;
            $item->paid = $childrenTotal;
    
            return $item;
        });
        
        $response = [
            'success'=>true,
            'cashtransaction'  =>$data,

        ];
        
        return response($response,200);
    }

    public function getCashType(Request $request)
    {
        $data = Cashtransaction::where('transaction_category', $request->type)
            ->with('subcashtransaction', 'from', 'to', 'subcashtransaction.akun')
            ->get();

        if (!empty($request->start_date) && !empty($request->end_date)) {
            $request->start_date = date('Y-m-d',strtotime($request->start_date));
            $request->end_date = date('Y-m-d',strtotime($request->end_date));
            $data = $data->whereBetween('date',[$request->start_date,$request->end_date]);
        }

        $filteredData = $data->filter(function ($value) {
            $total = $value->to ? $value->cashin : $value->cashout;

            $childrenout = Cashtransaction::where('pay_debt_to', $value->id)
                ->with('subcashtransaction', 'from', 'to', 'subcashtransaction.akun')
                ->get();

            $childrenin = Cashtransaction::where('receivable_id', $value->id)
                ->with('subcashtransaction', 'from', 'to', 'subcashtransaction.akun')
                ->get();

            $children = $childrenout->isNotEmpty() ? $childrenout : $childrenin;

            $childrenTotal = 0;
            foreach ($children as $child) {
                $child["total"] = $child->to ? $child->cashin : $child->cashout;
                $childrenTotal += $child["total"];
            }

            $remainingTotal = $total - $childrenTotal;
            
            $value['total'] = $remainingTotal;
            $value['children'] = $children;

            return $remainingTotal >= 1;
        })->values();

        $response = [
            'success' => true,
            'cashtransaction' => $filteredData,
        ];

        return response($response, 200);
    }

    public function getCashSaldoAwal(Request $request){

        $akun = Akun::where('name','Kas dan Bank')->first();
        $cash = Akun::where('perent_id',$akun->id)->get();
        $data = Cashtransaction::where('transaction_category','saldo_awal')
            ->whereHas('subcashtransaction', function($query) use ($cash) {
                $query->whereIn('akun_id', $cash->pluck('id'));
            })
            ->orderBy('id', 'desc');

        if (!empty($request->start_date) &&!empty($request->end_date)) {
            $request->start_date = date('Y-m-d',strtotime($request->start_date));
            $request->end_date = date('Y-m-d',strtotime($request->end_date));
            $data = $data->whereBetween('date',[$request->start_date,$request->end_date]);
        }

        $data = $data->with('subcashtransaction.akun')->get();

        $response = [
           'success' => true,
            'cashtransaction' => $data,
            'total' => $data->sum('cashin') ? $data->sum('cashin') : 0
        ];

        return response($response, 200);
    }

    public function getListCashIn(Request $request){

        $data = Cashtransaction::whereNotNull('transaction_category');

        if (!empty($request->start_date) && !empty($request->end_date)) {
            $request->start_date = date('Y-m-d', strtotime($request->start_date));
            $request->end_date = date('Y-m-d', strtotime($request->end_date));
            $data = $data->whereBetween('date', [$request->start_date, $request->end_date]);
        }

        $data = $data->whereIn('transaction_category', [
                'pendapatan',
                'pendapatan_jasa',
                'pendapatan_penjualan',
                'pendapatan_item_masuk',
                'pendapatan_retur_out',
                'pendapatan_retur_in',
                'potongan_pembelian',
                'apresiasi_aset',
            ])
            ->orWhere(function ($query) {
                $query->where('transaction_category', 'jual_aset')
                    ->where('sell_asets', 'laba');
            })
            ->orderBy('id', 'desc')
            ->with('subcashtransaction.akun', 'stocktransaction')
            ->get();
    
        // Map data and return transaction_type
        $data = $data->map(function ($item, $key) {
            $newData = [
                'transaction_name' => $item->subcashtransaction->map(function ($item2) use ($item) {
                    if ($item->id == $item2->cashtransaction_id) {
                        return $item2->akun->name;
                    }
                })[0]
            ];

            if ($item->transaction_category === 'pendapatan_retur_in' && $item->stocktransaction) {
                $item->stocktransaction->total = -abs($item->stocktransaction->total);
            }
    
            // Merge with existing data
            return array_merge(
                json_decode($item, true), 
                $newData
            );
        });

        $response = [
            'success' => true,
            'cashtransaction' => $data,
        ];
    
        return response($response, 200);
    }
    
    public function getListCashOut(Request $request){

        $data = Cashtransaction::whereNotNull('transaction_category');

        if (!empty($request->start_date) && !empty($request->end_date)) {
            $request->start_date = date('Y-m-d', strtotime($request->start_date));
            $request->end_date = date('Y-m-d', strtotime($request->end_date));
            $data = $data->whereBetween('date', [$request->start_date, $request->end_date]);
        }

        $data = $data->whereIn('transaction_category', [
                'biaya',
                'biaya_item_keluar',
                'penyusutan_aset',
                'biaya_retur_out',
            ])
            ->orWhere(function ($query) {
                $query->where('transaction_category', 'jual_aset')
                    ->where('sell_asets', 'rugi');
            })
            ->orderBy('id', 'desc')
            ->with('subcashtransaction.akun', 'stocktransaction', 'contact')
            ->get();
    
        // Map data and return transaction_type
        $data = $data->map(function ($item, $key) {
            $newData = [
                'transaction_name' => $item->subcashtransaction->map(function ($item2) use ($item) {
                    if ($item->id == $item2->cashtransaction_id) {
                        return $item2->akun->name;
                    }
                })[0]
            ];
    
            // Merge with existing data
            return array_merge(
                json_decode($item, true), 
                $newData
            );
        });

        $response = [
            'success' => true,
            'cashtransaction' => $data,
        ];
    
        return response($response, 200);
    }

    public function getListCashCash(Request $request)
    {
        $cashtransaction = Cashtransaction::whereNotNull('from')
            ->orWhereNotNull('to');
            
        $credit = Credit::where(function($query) {
                $query->where('cashout_id', '!=', '20')
                    ->orWhere('cashin_id', '!=', '51');
            });

        $akun = Akun::where('name','Kas dan Bank')->first();
        $cash = Akun::where('perent_id',$akun->id)->get();

        $saldoawal = Cashtransaction::where('transaction_category','saldo_awal')
            ->whereHas('subcashtransaction', function($query) use ($cash) {
                $query->whereIn('akun_id', $cash->pluck('id'));
            });

        if (!empty($request->start_date) && !empty($request->end_date)) {
            $start_date = date('Y-m-d', strtotime($request->start_date));
            $end_date = date('Y-m-d', strtotime($request->end_date));
            $cashtransaction->whereBetween('date', [$start_date, $end_date]);
            $credit->whereBetween('date', [$start_date, $end_date]);
            $saldoawal->whereBetween('date', [$start_date, $end_date]);
        }

        $cashTransactions = $cashtransaction->with('froms', 'tos', 'subcashtransaction')->get();
        $credits = $credit->with('cashin', 'cashout', 'stocktransaction')->get();
        $saldoawal = $saldoawal->with('subcashtransaction.akun')->get();
        Log::info('saldoawal' . $saldoawal);

        // Proses data cashTransactions dengan map
        $cashTransactionsResults = $cashTransactions->map(function ($data) {
            $category = 'Unknown';
            if ($data->transaction_category == 'tambah_modal') {
                $category = 'Tambah Modal';
            } elseif ($data->transaction_category == 'pendapatan') {
                $category = 'Catat Pendapatan';
            } elseif ($data->transaction_category == 'hutang') {
                $category = 'Catat Hutang';
            } elseif ($data->transaction_category == 'pembayaran_piutang') {
                $category = 'Pembayaran Piutang';
            } elseif ($data->transaction_category == 'jual_aset') {
                $category = 'Jual Aset';
            } elseif ($data->transaction_category == 'biaya') {
                $category = 'Biaya';
            } elseif ($data->transaction_category == 'bayar_hutang') {
                $category = 'Bayar Hutang';
            } elseif ($data->transaction_category == 'catat_piutang') {
                $category = 'Catat Piutang';
            } elseif ($data->transaction_category == 'tarik_laba') {
                $category = 'Tarik Laba';
            } elseif ($data->transaction_category == 'tarik_modal') {
                $category = 'Tarik Modal';
            } elseif ($data->transaction_category == 'beli_aset') {
                $category = 'Beli Aset';
            } elseif ($data->transaction_category == 'transfer_masuk') {
                $category = 'Transfer Kas';
            }

            return [
                'id' => $data->id,
                'cash_id' => $data->froms ?: $data->tos,
                'category' => $category,
                'date' => $data->date,
                'staff' => $data->staff,
                'desc' => $data->desc,
                'cashout' => $data->cashout ?: 0,
                'cashin' => $data->cashin ?: 0,
                'transfer' => $data->transfer ?: 0,
            ];
        });

        // Proses data credits dengan map
        $creditsResults = $credits->map(function ($data) {
            $category = 'Unknown';
            $stockTransaction = $data->stocktransaction;

            if ($stockTransaction->cashin_id && $stockTransaction->pending && is_null($stockTransaction->return)) {
                $category = 'Pesanan Penjualan';
            } elseif ($stockTransaction->cashout_id && $stockTransaction->pending && is_null($stockTransaction->return)) {
                $category = 'Pesanan Pembelian';
            } elseif ($stockTransaction->cashin_id && !is_null($stockTransaction->return) && $stockTransaction->pending) {
                $category = 'Return Penjualan';
            } elseif ($stockTransaction->cashout_id && !is_null($stockTransaction->return) && $stockTransaction->pending) {
                $category = 'Return Pembelian';
            } elseif ($stockTransaction->cashin_id && is_null($stockTransaction->return) && is_null($stockTransaction->pending)) {
                $category = 'Penjualan';
            } elseif ($stockTransaction->cashout_id && is_null($stockTransaction->return) && is_null($stockTransaction->pending)) {
                $category = 'Pembelian';
            }

            return [
                'id' => $data->stockTransaction->id,
                'cash_id' => $data->cashin ?: $data->cashout,
                'category' => $category,
                'date' => $data->date,
                'staff' => $data->staff,
                'desc' => $data->stocktransaction->description,
                'cashout' => $data->cashout_id ? $data->total : 0,
                'cashin' => $data->cashin_id ? $data->total : 0,
                'transfer' => 0,
            ];
        });

        // Get initial balance transactions
        $saldoawal = $saldoawal->map(function ($data) {
            return [
                'id' => null,
                'cash_id' => $data->subcashtransaction->first()->akun,
                'category' => 'Saldo Awal',
                'date' => $data->date,
                'staff' => $data->staff,
                'desc' => $data->desc,
                'cashout' => $data->cashout ?: 0,
                'cashin' => $data->cashin ?: 0,
                'transfer' => $data->transfer ?: 0,
            ];
        });

        // Gabungkan hasil map
        $results = collect($cashTransactionsResults)
            ->merge(collect($creditsResults))
            ->merge(collect($saldoawal));

        $response = [
            'success' => true,
            'data' => $results,
        ];

        return response($response, 200);
    }

    public function getCashTransfer(Request $request){
        $data = Cashtransaction::whereNotNull('transfer')->whereNotNull('from')->whereNotNull('to');
        if (!empty($request->start_date) && !empty($request->end_date)) {
            $request->start_date = date('Y-m-d',strtotime($request->start_date));
            $request->end_date = date('Y-m-d',strtotime($request->end_date));
            $data = $data->whereBetween('date',[$request->start_date,$request->end_date]);
        }
        // else{
        //     $data = $data->whereBetween('date',[date('Y-m-01',time()),date('Y-m-d',time())]);
        // }
        $data=$data->with('from','to')->orderBy('id', 'desc')->get();
        
        $response = [
            'success'=>true,
            'cashtransaction'  =>$data,

        ];
        
        return response($response,200);
    }

    public function getCashTransactionDetail(Request $request){
        $data = Cashtransaction::where('id',$request->id)
            ->with('from','to', 'contact', 'subcashtransaction.akun','credit','credit.isfrom','credit.isto')
            ->get();

        $response = [
            'success'=>true,
            'cashtransaction'  =>$data,

        ];
        
        return response($response,200);
    }

    // Dalam proses pembuatan untuk menu laporan kas
    public function getCashAndCredit(Request $request)
    {
        // Mengambil data dari tabel cashtransaction dan melakukan filter
        $cash = Cashtransaction::where(function($query) {
            $query->whereNotNull('to')
                ->orWhereNotNull('from');
        })->with('from', 'to', 'subcashtransaction.akun')->get();

        // Mengambil data dari tabel credit dan melakukan filter
        $credit = Credit::where(function($query) {
            $query->whereNotNull('cashin_id')
                ->orWhereNotNull('cashout_id');
        })->with('stocktransaction', 'cashin', 'cashout')->get();

        $data = [];

        foreach ($cash as $cashItem) {
            $cashItem->type_transaction = $cashItem->transaction_category; 
            $data[] = $cashItem;
        }

        foreach ($credit as $creditItem) {
            if (!is_null($creditItem->stocktransaction->return)) {
                $creditItem->type_transaction = $creditItem->stocktransaction->type == 'out' ? 'retur_pembelian' : 'retur_penjualan';
            } 
            elseif (!is_null($creditItem->stocktransaction->pending)) {
                if (!is_null($creditItem->cashout_id)) {
                    $creditItem->type_transaction = 'pesanan_pembelian';
                } elseif (!is_null($creditItem->cashin_id)) {
                    $creditItem->type_transaction = 'pesanan_penjualan';
                }
            } 
            elseif (is_null($creditItem->stocktransaction->return) && is_null($creditItem->stocktransaction->pending)) {
                if (!is_null($creditItem->cashout_id)) {
                    $creditItem->type_transaction = 'pembelian';
                } elseif (!is_null($creditItem->cashin_id)) {
                    $creditItem->type_transaction = 'penjualan';
                }
            }

            $data[] = $creditItem;
        }

        $response = [
            'success' => true,
            'cashcredittransaction' => $data,
        ];

        return response($response, 200);
    }

    public function getCashStockTo(Request $request){

        $data = Cashtransaction::with('to','subcashtransaction.akun','stocktransaction')->where('stocktransaction_id',$request->id)->where('transaction_category','=','pengembalian_uang_muka');

        if (!empty($request->start_date) && !empty($request->end_date)) {
            $request->start_date = date('Y-m-d',strtotime($request->start_date));
            $request->end_date = date('Y-m-d',strtotime($request->end_date));
            $data = $data->whereBetween('date',[$request->start_date,$request->end_date]);
        }

        $data =$data->get();
        
        $response = [
            'success'=>true,
            'cashtransaction'=>$data,
        ];
        
        return response($response,200);
    }
    
    public function getCashStockFrom(Request $request){

        $data = Cashtransaction::with('from','subcashtransaction.akun','stocktransaction')->where('stocktransaction_id',$request->id)->where('transaction_category','=','pengembalian_hutang_pesanan_penjualan');

        if (!empty($request->start_date) && !empty($request->end_date)) {
            $request->start_date = date('Y-m-d',strtotime($request->start_date));
            $request->end_date = date('Y-m-d',strtotime($request->end_date));
            $data = $data->whereBetween('date',[$request->start_date,$request->end_date]);
        }

        $data =$data->get();
        
        $response = [
            'success'=>true,
            'cashtransaction'=>$data,
        ];
        
        return response($response,200);
    }

    public function getCashFlow(Request $request) {
        $year = !empty($request->year) ? $request->year : date('Y');
        $previousYear = $year - 1;
    
        $start_date = date('Y-m-d', strtotime("{$year}-01-01"));
        $end_date = date('Y-m-d', strtotime("{$year}-12-31"));
        $prev_start_date = date('Y-m-d', strtotime("{$previousYear}-01-01"));
        $prev_end_date = date('Y-m-d', strtotime("{$previousYear}-12-31"));
    
        // Helper function to calculate sums
        $calculateSums = function($start_date, $end_date) {
            $pembelian = Credit::whereNotNull('cashout_id')
                ->where('cashout_id', '!=', '20')
                ->whereHas('stocktransaction', function ($stock) {
                    $stock->whereNotNull('cashout_id')->whereNull('nonmoney')->whereNull('return');
                })->whereBetween('date', [$start_date, $end_date])->sum('total');
    
            $penjualan = Credit::whereNotNull('cashin_id')
                ->where('cashin_id', '!=', '51')
                ->whereHas('stocktransaction', function ($stock) {
                    $stock->whereNotNull('cashin_id')->whereNull('nonmoney')->whereNull('return');
                })->whereBetween('date', [$start_date, $end_date])->sum('total');
            
            $returnpembelian = Credit::whereNotNull('cashin_id')
                ->whereHas('stocktransaction', function ($stock) {
                    $stock->whereNotNull('cashin_id')->whereNotNull('return');
                })->whereBetween('date', [$start_date, $end_date])->sum('total');

            $returnpenjualan = Credit::whereNotNull('cashout_id')
                ->whereHas('stocktransaction', function ($stock) {
                    $stock->whereNotNull('cashout_id')->whereNotNull('return');
                })->whereBetween('date', [$start_date, $end_date])->sum('total');
    
            $refundpesananpembelian = Cashtransaction::where('transaction_category', '=', 'refund_pesanan_pembelian')
                ->whereBetween('date', [$start_date, $end_date])->sum('cashin');
    
            $refundpesananpenjualan = Cashtransaction::where('transaction_category', '=', 'refund_pesanan_penjualan')
                ->whereBetween('date', [$start_date, $end_date])->sum('cashout');
    
            $pemngembalianuangmuka = Cashtransaction::where('transaction_category', '=', 'pengembalian_uang_muka')
                ->whereBetween('date', [$start_date, $end_date])->sum('cashin');
    
            $pengembalianhutangpesananpenjualan = Cashtransaction::where('transaction_category', '=', 'pengembalian_hutang_pesanan_penjualan')
                ->whereBetween('date', [$start_date, $end_date])->sum('cashout');
    
            $pendapatan = Cashtransaction::where('transaction_category', '=', 'pendapatan')
                ->whereBetween('date', [$start_date, $end_date])->sum('cashin');
    
            $biaya = Cashtransaction::where('transaction_category', '=', 'biaya')
                ->whereBetween('date', [$start_date, $end_date])->sum('cashout');
    
            $jualaset = Cashtransaction::where('transaction_category', '=', 'jual_aset')
                ->whereBetween('date', [$start_date, $end_date])->sum('cashin');
    
            $beliaset = Cashtransaction::where('transaction_category', '=', 'beli_aset')
                ->whereBetween('date', [$start_date, $end_date])->sum('cashout');
    
            $tambahmodal = Cashtransaction::where('transaction_category', '=', 'tambah_modal')
                ->whereBetween('date', [$start_date, $end_date])->sum('cashin');
    
            $tarikmodal = Cashtransaction::where('transaction_category', '=', 'tarik_modal')
                ->whereBetween('date', [$start_date, $end_date])->sum('cashout');
    
            $tariklaba = Cashtransaction::where('transaction_category', '=', 'tarik_laba')
                ->whereBetween('date', [$start_date, $end_date])->sum('cashout');
    
            $hutang = Cashtransaction::where('transaction_category', '=', 'hutang')
                ->whereBetween('date', [$start_date, $end_date])->sum('cashin');
    
            $bayarhutang = Cashtransaction::where('transaction_category', '=', 'bayar_hutang')
                ->whereBetween('date', [$start_date, $end_date])->sum('cashout');
    
            $piutang = Cashtransaction::where('transaction_category', '=', 'catat_piutang')
                ->whereBetween('date', [$start_date, $end_date])->sum('cashout');
    
            $pembayaranpiutang = Cashtransaction::where('transaction_category', '=', 'pembayaran_piutang')
                ->whereBetween('date', [$start_date, $end_date])->sum('cashin');
    
            return [
                'operasi' => [
                    'data' => [
                        ['name' => 'Penerimaan Kas Dari Pelanggan', 'total' => $penjualan],
                        ['name' => 'Pendapatan Lain', 'total' => $pendapatan],
                        ['name' => 'Penerimaan Kas dari Retur Pembelian', 'total' => $returnpembelian],
                        ['name' => 'Penerimaan Kas dari Pembatalan Pesanan', 'total' => $refundpesananpembelian],
                        ['name' => 'Penerimaan Kas dari Pengembalian Dana Supplier', 'total' => $pemngembalianuangmuka],
                        ['name' => 'Pembayaran Kepada Supplier', 'total' => $pembelian],
                        ['name' => 'Biaya', 'total' => $biaya],
                        ['name' => 'Pengeluaran Kas untuk Retur Penjualan', 'total' => $returnpenjualan],
                        ['name' => 'Pengeluaran Kas atas Pesanan Yang Tidak Dapat Dipenuhi', 'total' => $refundpesananpenjualan],
                        ['name' => 'Pengeluaran Kas untuk Penyesuaian Harga Penjualan', 'total' => $pengembalianhutangpesananpenjualan],
                    ],
                    'total' => $penjualan + $pendapatan + $returnpembelian + $refundpesananpembelian + $pemngembalianuangmuka - $pembelian - $biaya - $returnpenjualan - $refundpesananpenjualan - $pengembalianhutangpesananpenjualan,
                ],
                'investasi' => [
                    'data' => [
                        ['name' => 'Jual Aset', 'total' => $jualaset],
                        ['name' => 'Beli Aset', 'total' => $beliaset],
                    ],
                    'total' => $jualaset - $beliaset,
                ],
                'pendanaan' => [
                    'data' => [
                        ['name' => 'Penerimaan Modal', 'total' => $tambahmodal],
                        ['name' => 'Penerimaan Pinjaman', 'total' => $hutang],
                        ['name' => 'Pembayaran Piutang Oleh Debitur', 'total' => $pembayaranpiutang],
                        ['name' => 'Distribusi Laba Kepada Investor', 'total' => $tariklaba],
                        ['name' => 'Penarikan Modal Oleh Investor', 'total' => $tarikmodal],
                        ['name' => 'Pemberian Pinjaman Kepada Debitur', 'total' => $piutang],
                        ['name' => 'Pembayaran Utang', 'total' => $bayarhutang],
                    ],
                    'total' => $tambahmodal + $hutang + $pembayaranpiutang - $tariklaba - $tarikmodal - $piutang - $bayarhutang,
                ],
            ];
        };
    
        $currentYearData = $calculateSums($start_date, $end_date);
        $previousYearData = $calculateSums($prev_start_date, $prev_end_date);
    
        return response([
            'success' => true,
            'currentYear' => $currentYearData,
            'previousYear' => $previousYearData,
        ], 200);
    }    
   
    public function createCashIn(Request $request){
        $request->validate([
            'to' =>'required',
            'keterangan' =>'nullable',
            'desc' =>'nullable',
            'staff' =>'nullable',
            'transaction_id' =>'nullable',
            'date' =>'required',
            'type' => 'required',
            'akun_id.*' =>'required',
            'total.*'  =>'required|numeric',
        ]);   
        
        // Pengecekan tambahan untuk memastikan request lengkap
        if (!$request->has(['to', 'keterangan', 'desc', 'staff', 'date', 'type', 'akun_id', 'total'])) {
            return response()->json(['message' => $request->all()], 400);
        }

        $uang = $request->uang;
        $cash = new Cashtransaction;
        $cash->to = $request->to;
        $cash->staff = $request->staff;
        $cash->desc = $request->keterangan;
        $cash->date = date("Y-m-d", strtotime($request->date));
        $cash->transaction_category = $request->type;
        $cash->receivable_id = $request->transaction_id;
        // add trx id if jual aset
        if($request->type === 'jual_aset' || $request->type === 'penyusutan_aset' || $request->type === 'apresiasi_aset'){
            $akId = $request->akun_id[0];
            $cId = Subcashtransaction::where('akun_id', $akId)->first();
            $cash->transaction_ref = $cId->cashtransaction_id;
        }
        $cash->cashin = 0;
        $cash->save();

        // switch case berdasarkan type
        switch ($request->type) {
            case 'modal_aset':
                $sub = new Subcashtransaction;
                $sub->cashtransaction_id = $cash->id;
                $sub->akun_id = $request->to;
                $sub->desc = null;
                $sub->total = $request->total[0];
                $sub->save();
                break;

            case 'penyusutan_aset':
            case 'apresiasi_aset':
                $sub = new Subcashtransaction;
                $sub->cashtransaction_id = $cash->id;
                $sub->akun_id = $request->to;
                $sub->desc = null;
                $sub->total = $request->uang;
                $sub->save();
                break;
            
            default:
                break;
        }

        $data = $request->akun_id;
        $total = 0;
        function rekursif($akun,$sub,$name){
            foreach ($akun as $key => $value) {
                if (!empty($value->children)) {
                    rekursif($value->children,$sub,$name);
                }
                if ($value->name==$name->name) {
                    $sub->total = -1*$sub->total;
                }
            }
        }
        foreach ( $data as $key => $value) {
            $sub = new Subcashtransaction;
            $sub->cashtransaction_id = $cash->id;
            $sub->akun_id = $request->akun_id[$key];
            if($request->type === 'jual_aset' || $request->type === 'penyusutan_aset' || $request->type === 'apresiasi_aset'){
                $sub->desc = $request->desc;
            } else {
                $sub->desc = $request->desc[$key];
            }
            
            $sub->total = $request->total[$key];
            $akun = Akun::where('name','Harta')->with(str_repeat('children.',10))->get();
            $nama = Akun::find($sub->akun_id);

            rekursif($akun,$sub,$nama);
            $sub->save();

            $subtransaction[]= $sub;
            $akun = Akun::find($sub->akun_id);
            $akun->total = $akun->total + $request->total[$key];
            $akun->save();

            $total = $total + $request->total[$key];

            if ($request->type === 'jual_aset') {
                if ($akun->id === 115) {
                    $cash = Cashtransaction::find($cash->id);
                    $cash->sell_asets = 'rugi';
                    $cash->save();
                } else {
                    $cash = Cashtransaction::find($cash->id);
                    $cash->sell_asets = 'laba';
                    $cash->save();
                }
            }
            
        }

        
        $cash = Cashtransaction::find($cash->id);
        $cash->cashin = $uang ? $uang : $cash->cashin + $total;
        $cash->cashout = 0 ; 
        $cash->transfer = 0 ; 
        $cash->save();
        
        $akun = Akun::find($request->to);
        $akun->total = $akun->total + $total;
        $akun->save();

        $response = [
            'success'=>true,
            'cashtransaction'  => $cash,
            'subcashtransaction'  =>$subtransaction,
        ];

        return response($response,200);
    }

    public function createCashOut(Request $request){
        $request->validate([
            'from' => [
                Rule::requiredIf(!in_array($request->type, ['biaya', 'beli_aset'])),
                'nullable'
            ],
            'keterangan' =>'nullable',
            'desc' => 'nullable',
            'staff' =>'nullable',
            'transaction_id' =>'nullable',
            'contact_id' =>'nullable',
            'jumlah_bayar' =>'nullable',
            'date' =>'required',
            'type' => 'required',
            'akun_id.*' =>'required',
            'total.*'  =>'required|numeric',
        ]);

        if (empty($request->from) && in_array($request->type, ['biaya', 'beli_aset'])) {
            $request->from = Akun::where('iscash',true)->first()->id;
        }

        $cash = new Cashtransaction;
        $cash->from = $request->from;
        $cash->staff = $request->staff;
        $cash->contact_id = $request->contact_id;
        $cash->desc = $request->keterangan;
        $cash->date = date("Y-m-d", strtotime($request->date));
        $cash->transaction_category = $request->type;
        $cash->pay_debt_to = $request->transaction_id;
        $cash->cashout = 0;
        $cash->save();

        $data = $request->akun_id;
        $total = 0;
        function rekursif($akun,$sub,$name){
            foreach ($akun as $key => $value) {
                if (!empty($value->children)) {
                    rekursif($value->children,$sub,$name);
                }
                if ($value->name==$name->name) {
                    $sub->total = -1*$sub->total;
                }
            }
        }
        foreach ( $data as $key => $value) {
            $sub = new Subcashtransaction;
            $sub->cashtransaction_id = $cash->id;
            $sub->akun_id = $request->akun_id[$key];
            $sub->total = $request->total[$key];
            $sub->desc = $request->desc[$key];
            $akun = Akun::where('name','Modal')->with(str_repeat('children.',10))->get();
            $akun2 = Akun::where('name','Kewajiban')->with(str_repeat('children.',10))->get();
            $nama = Akun::find($sub->akun_id);

            rekursif($akun,$sub,$nama);
            rekursif($akun2,$sub,$nama);
            $sub->save();

            $subtransaction[]= $sub;
            $akun = Akun::find($sub->akun_id);
            $akun->total = $akun->total + $request->total[$key];
            $akun->save();

            $total = $total + $request->total[$key];
            
        }

        $cash = Cashtransaction::find($cash->id);
        $cash->cashout = in_array($request->type, ['biaya', 'beli_aset']) ? $request->jumlah_bayar : $cash->cashout + $total;
        $cash->cashin = 0 ; 
        $cash->transfer = 0 ; 
        $cash->total = 0 ; 
        $cash->save();

        if (in_array($request->type, ['biaya', 'beli_aset'])) {
            $cash = Cashtransaction::find($cash->id);
            $cash->total = $cash->total + $total;
            $cash->save();

            $creditcash = new CreditCashTransaction;
            $creditcash->cashtransaction_id = $cash->id;
            $creditcash->from_id = $request->from;
            $creditcash->total = $cash->cashout;
            $creditcash->staff = $request->staff;
            $creditcash->date = date("Y-m-d", strtotime($request->date));
            $creditcash->save();
        }
        
        $akun = Akun::find($request->from);
        $akun->total = $akun->total - $total;
        $akun->save();

        $response = [
            'success'=>true,
            'cashtransaction'  => $cash,
            'subcashtransaction'  =>$subtransaction,
        ];

        return response($response,200);
    }

        public function createCashTransfer(Request $request){
        $request->validate([
            'to' =>'required',
            'from' =>'required',
            'date' =>'required',
            'type' => 'required',
            'desc' =>'nullable',
            'staff' =>'required',
            'total'  =>'required|numeric',
        ]);                

        $cash = new Cashtransaction;
        $cash->from = $request->from;
        $cash->to = $request->to;
        $cash->staff = $request->staff;
        $cash->desc = $request->desc;
        $cash->transaction_category = $request->type;
        $cash->date = date("Y-m-d", strtotime($request->date));
        $cash->transfer = $request->total;
        $cash->cashout = 0 ; 
        $cash->cashin = 0 ; 
        $cash->save();
        
        $akun = Akun::find($request->from);
        $akun->total = $akun->total - $request->total;
        $akun->save();

        $akun = Akun::find($request->to);
        $akun->total = $akun->total + $request->total;
        $akun->save();

        $response = [
            'success'=>true,
            'cashtransaction'  => $cash,
        ];

        return response($response,200);
    }

    public function deleteCashTransaction(Request $request){
        $ch = Cashtransaction::where("transaction_ref", $request->id)->first();
        if($ch){
            $response = [
                'success'=>false,
                'message'  => "Untuk menghapus transaksi ini, Anda harus menghapus transaksi no." . $ch->id . ", yang dapat Anda temukan di JUAL ASET, APRESIASI ASET, atau PENYUSUTAN ASET.",
            ];
            return response($response,400);
        }
        $checkcashout = Cashtransaction::where("pay_debt_to", $request->id)->first();
        if($checkcashout){
            $namecash = '';
            if($checkcashout->transaction_category == 'bayar_hutang') $namecash = 'BAYAR HUTANG';
            $response = [
                'success'=>false,
                'message'  => "Untuk menghapus transaksi ini, Anda harus menghapus transaksi no." . $checkcashout->id . ", yang dapat Anda temukan di " . $namecash . ".",
            ];
            return response($response,400);
        }
        $checkcashin = Cashtransaction::where("receivable_id", $request->id)->first();
        if($checkcashin){
            $namecash = '';
            if($checkcashin->transaction_category == 'pembayaran_piutang') $namecash = 'PEMBAYARAN PIUTANG';
            $response = [
                'success'=>false,
                'message'  => "Untuk menghapus transaksi ini, Anda harus menghapus transaksi no." . $checkcashin->id . ", yang dapat Anda temukan di " . $namecash . ".",
            ];
            return response($response,400);
        }
        $data = Cashtransaction::find($request->id);
        if ($data->cashin) {
            $sub = Subcashtransaction::where('cashtransaction_id',$request->id)->get();

            $akun = Akun::find($data->to);
            $akun->total = $akun->total - $data->cashin;
            $akun->save();

            foreach ($sub as $key => $value) {
                $akun = Akun::find($value->akun_id);
                $akun->total = $akun->total - $value->total;
                $akun->save(); 
           }
           Subcashtransaction::where('cashtransaction_id',$request->id)->delete();
        }elseif($data->cashout) {
            $sub = Subcashtransaction::where('cashtransaction_id',$request->id)->get();

            $akun = Akun::find($data->from);
            $akun->total = $akun->total + $data->cashout;
            $akun->save();
            foreach ($sub as $key => $value) {
                $akun = Akun::find($value->akun_id);
                $akun->total = $akun->total - $value->total;
                $akun->save(); 
            }
            Subcashtransaction::where('cashtransaction_id',$request->id)->delete();
        }elseif($data->transfer){
            $akun = Akun::find($data->from);
            $akun->total = $akun->total + $data->transfer;
            $akun->save();

            $akun = Akun::find($data->to);
            $akun->total = $akun->total - $data->transfer;
            $akun->save();
        }

        // Hapus credit_cashtransaction
        CreditCashTransaction::where('cashtransaction_id', $request->id)->delete();
        
        $data->delete();
        $response = [
            'success'=>true,
            'cashtransaction'  => $data,
        ];
 
        return response($response,200);
    }

}